import sys
import yaml


def main():
    # various smoke tests on an installed PyYAML with extension
    if not getattr(yaml, '_yaml', None):
        raise Exception('C extension is not available at `yaml._yaml`')

    print('embedded libyaml version is {0}'.format(yaml._yaml.get_version_string()))

    for loader, dumper in [(yaml.CLoader, yaml.CDumper), (yaml.Loader, yaml.Dumper)]:
        testyaml = 'dude: mar'
        loaded = yaml.load(testyaml, Loader=loader)
        dumped = yaml.dump(loaded, Dumper=dumper)
        if testyaml != dumped.strip():
            raise Exception('roundtrip failed with {0}/{1}'.format(loader, dumper))
    print('smoke test passed for {0}'.format(sys.executable))


if __name__ == '__main__':
    main()


import inspect


def _bridge_build_meta():
    import functools
    import sys

    from setuptools import build_meta

    self_module = sys.modules[__name__]

    for attr_name in build_meta.__all__:
        attr_value = getattr(build_meta, attr_name)
        if callable(attr_value):
            setattr(self_module, attr_name, functools.partial(_expose_config_settings, attr_value))


class ActiveConfigSettings:
    _current = {}

    def __init__(self, config_settings):
        self._config = config_settings

    def __enter__(self):
        type(self)._current = self._config

    def __exit__(self, exc_type, exc_val, exc_tb):
        type(self)._current = {}

    @classmethod
    def current(cls):
        return cls._current


def _expose_config_settings(real_method, *args, **kwargs):
    from contextlib import nullcontext
    import inspect

    sig = inspect.signature(real_method)
    boundargs = sig.bind(*args, **kwargs)

    config = boundargs.arguments.get('config_settings')

    ctx = ActiveConfigSettings(config) if config else nullcontext()

    with ctx:
        return real_method(*args, **kwargs)


_bridge_build_meta()





import yaml, test_emitter

def test_loader_error(error_filename, verbose=False):
    try:
        with open(error_filename, 'rb') as file:
            list(yaml.load_all(file, yaml.FullLoader))
    except yaml.YAMLError as exc:
        if verbose:
            print("%s:" % exc.__class__.__name__, exc)
    else:
        raise AssertionError("expected an exception")

test_loader_error.unittest = ['.loader-error']

def test_loader_error_string(error_filename, verbose=False):
    try:
        with open(error_filename, 'rb') as file:
            list(yaml.load_all(file.read(), yaml.FullLoader))
    except yaml.YAMLError as exc:
        if verbose:
            print("%s:" % exc.__class__.__name__, exc)
    else:
        raise AssertionError("expected an exception")

test_loader_error_string.unittest = ['.loader-error']

def test_loader_error_single(error_filename, verbose=False):
    try:
        with open(error_filename, 'rb') as file:
            yaml.load(file.read(), yaml.FullLoader)
    except yaml.YAMLError as exc:
        if verbose:
            print("%s:" % exc.__class__.__name__, exc)
    else:
        raise AssertionError("expected an exception")

test_loader_error_single.unittest = ['.single-loader-error']

def test_emitter_error(error_filename, verbose=False):
    with open(error_filename, 'rb') as file:
        events = list(yaml.load(file, Loader=test_emitter.EventsLoader))
    try:
        yaml.emit(events)
    except yaml.YAMLError as exc:
        if verbose:
            print("%s:" % exc.__class__.__name__, exc)
    else:
        raise AssertionError("expected an exception")

test_emitter_error.unittest = ['.emitter-error']

def test_dumper_error(error_filename, verbose=False):
    with open(error_filename, 'rb') as file:
        code = file.read()
    try:
        import yaml
        from io import StringIO
        exec(code)
    except yaml.YAMLError as exc:
        if verbose:
            print("%s:" % exc.__class__.__name__, exc)
    else:
        raise AssertionError("expected an exception")

test_dumper_error.unittest = ['.dumper-error']

if __name__ == '__main__':
    import test_appliance
    test_appliance.run(globals())




# pytest custom collection adapter for legacy pyyaml unit tests/data files; surfaces each
# legacy test case as a pyyaml item

import os
import pathlib
import pytest
import warnings

from test_appliance import find_test_filenames, DATA

try:
    from yaml import _yaml
    HAS_LIBYAML_EXT = True
    del _yaml
except ImportError:
    HAS_LIBYAML_EXT = False


_test_filenames = find_test_filenames(DATA)

# ignore all datafiles
collect_ignore_glob = ['data/*']


class PyYAMLItem(pytest.Item):
    def __init__(self, parent=None, config=None, session=None, nodeid=None, function=None, filenames=None, **kwargs):
        self._function = function
        self._fargs = filenames or []

        super().__init__(os.path.basename(filenames[0]) if filenames else parent.name, parent, config, session, nodeid)
        # this is gnarly since the type of fspath is private; fixed in pytest 7 to use pathlib on the `path` attr
        if filenames:  # pass the data file location as the test path
            self.fspath = parent.fspath.__class__(filenames[0])
            self.lineno = 1
        else:  # pass the function location in the code
            self.fspath = parent.fspath.__class__(function.__code__.co_filename)
            self.lineno = function.__code__.co_firstlineno

    def runtest(self):
        self._function(verbose=True, *self._fargs)

    def reportinfo(self):
        return self.fspath, self.lineno, ''


class PyYAMLCollector(pytest.Collector):
    def __init__(self, name, parent=None, function=None, **kwargs):
        self._function = function
        self.fspath = parent.fspath.__class__(function.__code__.co_filename)
        self.lineno = function.__code__.co_firstlineno

        # avoid fspath deprecation warnings on pytest < 7
        if hasattr(self, 'path') and 'fspath' in kwargs:
            del kwargs['fspath']

        super().__init__(name=name, parent=parent, **kwargs)

    def collect(self):
        items = []

        unittest = getattr(self._function, 'unittest', None)

        if unittest is True:  # no filenames
            items.append(PyYAMLItem.from_parent(parent=self, function=self._function, filenames=None))
        else:
            for base, exts in _test_filenames:
                filenames = []
                for ext in unittest:
                    if ext not in exts:
                        break
                    filenames.append(os.path.join(DATA, base + ext))
                else:
                    skip_exts = getattr(self._function, 'skip', [])
                    for skip_ext in skip_exts:
                        if skip_ext in exts:
                            break
                    else:
                        items.append(PyYAMLItem.from_parent(parent=self, function=self._function, filenames=filenames))

        return items or None

    def reportinfo(self):
        return self.fspath, self.lineno, ''

    @classmethod
    def from_parent(cls, parent, fspath, **kwargs):
        return super().from_parent(parent=parent, fspath=fspath, **kwargs)


@pytest.hookimpl(hookwrapper=True, trylast=True)
def pytest_pycollect_makeitem(collector, name: str, obj: object):
    outcome = yield
    outcome.get_result()
    if not callable(obj):
        outcome.force_result(None)
        return
    unittest = getattr(obj, 'unittest', None)

    if not unittest:
        outcome.force_result(None)
        return

    if unittest is True:  # no file list to run against, just return a test item instead of a collector
        outcome.force_result(PyYAMLItem.from_parent(name=name, parent=collector, fspath=collector.fspath, function=obj))
        return

    # there's a file list; return a collector to create individual items for each
    outcome.force_result(PyYAMLCollector.from_parent(name=name, parent=collector, fspath=collector.fspath, function=obj))
    return


def pytest_collection_modifyitems(session, config, items):
    pass


def pytest_ignore_collect(collection_path: pathlib.Path):
    basename = collection_path.name
    # ignore all Python files in this subtree for normal pytest collection
    if basename not in ['test_yaml.py', 'test_yaml_ext.py']:
        return True

    # ignore extension tests (depending on config)
    if basename == 'test_yaml_ext.py':
        require_libyaml = os.environ.get('PYYAML_FORCE_LIBYAML', None)
        if require_libyaml == '1' and not HAS_LIBYAML_EXT:
            raise RuntimeError('PYYAML_FORCE_LIBYAML envvar is set, but libyaml extension is not available')
        if require_libyaml == '0':
            return True
        if not HAS_LIBYAML_EXT:
            warnings.warn('libyaml extension is not available, skipping libyaml tests')
            return True

